@@ -38,7 +38,7 @@
 #endif
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.219 2012/05/09 23:20:56 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.222 2012/06/25 16:34:58 tg Exp $");
 
 #if HAVE_KILLPG
 /*
@@ -2407,7 +2407,11 @@ c_set(const char **wp)
 	 * if there are no command substitutions).
 	 * Switched ksh (!posix !sh) to POSIX in mksh R39b.
 	 */
+#ifdef MKSH_LEGACY_MODE
+	return (subst_exstat);
+#else
 	return (Flag(FSH) ? subst_exstat : 0);
+#endif
 }
 
 int
@@ -2612,6 +2616,7 @@ c_exec(const char **wp MKSH_A_UNUSED)
 		for (i = 0; i < NUFILE; i++) {
 			if (e->savefd[i] > 0)
 				close(e->savefd[i]);
+#ifndef MKSH_LEGACY_MODE
 			/*
 			 * keep all file descriptors > 2 private for ksh,
 			 * but not for POSIX or legacy/kludge sh
@@ -2619,6 +2624,7 @@ c_exec(const char **wp MKSH_A_UNUSED)
 			if (!Flag(FPOSIX) && !Flag(FSH) && i > 2 &&
 			    e->savefd[i])
 				fcntl(i, F_SETFD, FD_CLOEXEC);
+#endif
 		}
 		e->savefd = NULL;
 	}
@@ -3592,11 +3598,6 @@ c_cat(const char **wp)
 	char *buf, *cp;
 #define MKSH_CAT_BUFSIZ 4096
 
-	if ((buf = malloc_osfunc(MKSH_CAT_BUFSIZ)) == NULL) {
-		bi_errorf(Toomem, (unsigned long)MKSH_CAT_BUFSIZ);
-		return (1);
-	}
-
 	/* parse options: POSIX demands we support "-u" as no-op */
 	while ((rv = ksh_getopt(wp, &builtin_opt, "u")) != -1) {
 		switch (rv) {
@@ -3611,6 +3612,11 @@ c_cat(const char **wp)
 	wp += builtin_opt.optind;
 	rv = 0;
 
+	if ((buf = malloc_osfunc(MKSH_CAT_BUFSIZ)) == NULL) {
+		bi_errorf(Toomem, (unsigned long)MKSH_CAT_BUFSIZ);
+		return (1);
+	}
+
 	do {
 		if (*wp) {
 			fn = *wp++;
