@@ -1,7 +1,7 @@
 /*	$OpenBSD: jobs.c,v 1.38 2009/12/12 04:28:44 deraadt Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -22,7 +22,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.81 2011/08/27 18:06:46 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.88 2012/05/04 22:34:50 tg Exp $");
 
 #if HAVE_KILLPG
 #define mksh_killpg		killpg
@@ -37,13 +37,15 @@ __RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.81 2011/08/27 18:06:46 tg Exp $");
 #define PSIGNALLED	2
 #define PSTOPPED	3
 
-typedef struct proc	Proc;
+typedef struct proc Proc;
 struct proc {
 	Proc *next;		/* next process in pipeline (if any) */
 	pid_t pid;		/* process id */
 	int state;
 	int status;		/* wait status */
-	char command[48];	/* process command string */
+	/* process command string from vistree */
+	char command[64 - (ALLOC_SIZE + sizeof(Proc *) + sizeof(pid_t) +
+	    2 * sizeof(int))];
 };
 
 /* Notify/print flag - j_print() argument */
@@ -87,7 +89,7 @@ struct job {
 	int32_t	age;		/* number of jobs started */
 	Coproc_id coproc_id;	/* 0 or id of coprocess output pipe */
 #ifndef MKSH_UNEMPLOYED
-	struct termios ttystat;	/* saved tty state for stopped jobs */
+	mksh_ttyst ttystat;	/* saved tty state for stopped jobs */
 	pid_t saved_ttypgrp;	/* saved tty process group for stopped jobs */
 #endif
 };
@@ -327,11 +329,13 @@ j_change(void)
 				kshpgrp = kshpid;
 			}
 		}
+#ifndef MKSH_DISABLE_TTY_WARNING
 		if (use_tty && !ttypgrp_ok)
 			warningf(false, "%s: %s", "warning",
 			    "won't have full job control");
+#endif
 		if (tty_fd >= 0)
-			tcgetattr(tty_fd, &tty_state);
+			mksh_tcget(tty_fd, &tty_state);
 	} else {
 		ttypgrp_ok = false;
 		if (Flag(FTALKING))
@@ -801,14 +805,14 @@ j_resume(const char *cp, int bg)
 		/* attach tty to job */
 		if (j->state == PRUNNING) {
 			if (ttypgrp_ok && (j->flags & JF_SAVEDTTY))
-				tcsetattr(tty_fd, TCSADRAIN, &j->ttystat);
+				mksh_tcset(tty_fd, &j->ttystat);
 			/* See comment in j_waitj regarding saved_ttypgrp. */
 			if (ttypgrp_ok &&
 			    tcsetpgrp(tty_fd, (j->flags & JF_SAVEDTTYPGRP) ?
 			    j->saved_ttypgrp : j->pgrp) < 0) {
 				rv = errno;
 				if (j->flags & JF_SAVEDTTY)
-					tcsetattr(tty_fd, TCSADRAIN, &tty_state);
+					mksh_tcset(tty_fd, &tty_state);
 				sigprocmask(SIG_SETMASK, &omask, NULL);
 				bi_errorf("%s %s(%d, %ld) %s: %s",
 				    "1st", "tcsetpgrp", tty_fd,
@@ -830,7 +834,7 @@ j_resume(const char *cp, int bg)
 		if (!bg) {
 			j->flags &= ~JF_FG;
 			if (ttypgrp_ok && (j->flags & JF_SAVEDTTY))
-				tcsetattr(tty_fd, TCSADRAIN, &tty_state);
+				mksh_tcset(tty_fd, &tty_state);
 			if (ttypgrp_ok && tcsetpgrp(tty_fd, kshpgrp) < 0)
 				warningf(true, "%s %s(%d, %ld) %s: %s",
 				    "fg: 2nd", "tcsetpgrp", tty_fd,
@@ -1060,6 +1064,9 @@ j_waitj(Job *j,
     const char *where)
 {
 	int rv;
+#ifdef MKSH_NO_SIGSUSPEND
+	sigset_t omask;
+#endif
 
 	/*
 	 * No auto-notify on the job we are waiting on.
@@ -1076,7 +1083,14 @@ j_waitj(Job *j,
 	while (j->state == PRUNNING ||
 	    ((flags & JW_STOPPEDWAIT) && j->state == PSTOPPED)) {
 #ifndef MKSH_NOPROSPECTOFWORK
+#ifdef MKSH_NO_SIGSUSPEND
+		sigprocmask(SIG_SETMASK, &sm_default, &omask);
+		pause();
+		/* note that handlers may run here so they need to know */
+		sigprocmask(SIG_SETMASK, &omask, NULL);
+#else
 		sigsuspend(&sm_default);
+#endif
 #else
 		j_sigchld(SIGCHLD);
 #endif
@@ -1118,7 +1132,7 @@ j_waitj(Job *j,
 				    (long)kshpgrp, "failed", strerror(errno));
 			if (j->state == PSTOPPED) {
 				j->flags |= JF_SAVEDTTY;
-				tcgetattr(tty_fd, &j->ttystat);
+				mksh_tcget(tty_fd, &j->ttystat);
 			}
 		}
 #endif
@@ -1134,9 +1148,9 @@ j_waitj(Job *j,
 			 */
 			if (j->state == PEXITED && j->status == 0 &&
 			    (j->flags & JF_USETTYMODE)) {
-				tcgetattr(tty_fd, &tty_state);
+				mksh_tcget(tty_fd, &tty_state);
 			} else {
-				tcsetattr(tty_fd, TCSADRAIN, &tty_state);
+				mksh_tcset(tty_fd, &tty_state);
 				/*-
 				 * Don't use tty mode if job is stopped and
 				 * later restarted and exits. Consider
@@ -1242,6 +1256,12 @@ j_sigchld(int sig MKSH_A_UNUSED)
 	pid_t pid;
 	int status;
 	struct rusage ru0, ru1;
+#ifdef MKSH_NO_SIGSUSPEND
+	sigset_t omask;
+
+	/* this handler can run while SIGCHLD is not blocked, so block it now */
+	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
+#endif
 
 #ifndef MKSH_NOPROSPECTOFWORK
 	/*
@@ -1253,7 +1273,7 @@ j_sigchld(int sig MKSH_A_UNUSED)
 	for (j = job_list; j; j = j->next)
 		if (j->ppid == procpid && !(j->flags & JF_STARTED)) {
 			held_sigchld = 1;
-			return;
+			goto j_sigchld_out;
 		}
 #endif
 
@@ -1270,7 +1290,7 @@ j_sigchld(int sig MKSH_A_UNUSED)
 		 * or interrupted (-1)
 		 */
 		if (pid <= 0)
-			return;
+			goto j_sigchld_out;
 
 		getrusage(RUSAGE_CHILDREN, &ru1);
 
@@ -1313,6 +1333,12 @@ j_sigchld(int sig MKSH_A_UNUSED)
 #else
 	    while (/* CONSTCOND */ 0);
 #endif
+
+ j_sigchld_out:
+#ifdef MKSH_NO_SIGSUSPEND
+	sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif
+	/* nothing */;
 }
 
 /*
@@ -1491,6 +1517,8 @@ j_print(Job *j, int how, struct shf *shf)
 				strlcpy(buf, sigtraps[WTERMSIG(p->status)].mess,
 				    sizeof(buf));
 			break;
+		default:
+			buf[0] = '\0';
 		}
 
 		if (how != JP_SHORT) {
