@@ -1,7 +1,8 @@
 /*	$OpenBSD: lex.c,v 1.45 2011/03/09 09:30:39 okan Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+ *		 2011, 2012
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -22,7 +23,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.158 2011/11/26 17:56:30 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.163 2012/05/04 22:44:33 tg Exp $");
 
 /*
  * states while lexing word
@@ -1382,6 +1383,7 @@ getsc_line(Source *s)
 	}
 	if (interactive)
 		change_winsz();
+#ifndef MKSH_NO_CMDLINE_EDITING
 	if (have_tty && (
 #if !MKSH_S_NOVI
 	    Flag(FVI) ||
@@ -1395,7 +1397,9 @@ getsc_line(Source *s)
 			nread = 0;
 		xp[nread] = '\0';
 		xp += nread;
-	} else {
+	} else
+#endif
+	  {
 		if (interactive)
 			pprompt(prompt, 0);
 		else
@@ -1440,25 +1444,6 @@ getsc_line(Source *s)
 		alarm(0);
 	}
 	cp = Xstring(s->xs, xp);
-#if !defined(MKSH_SMALL) && !defined(MKSH_DISABLE_DEPRECATED)
-	if (interactive && *cp == '!' && cur_prompt == PS1) {
-		int linelen;
-
-		linelen = Xlength(s->xs, xp);
-		XcheckN(s->xs, xp, Zfc_e_dash + /* NUL */ 1);
-		/* reload after potential realloc */
-		cp = Xstring(s->xs, xp);
-		/* change initial '!' into space */
-		*cp = ' ';
-		/* NUL terminate the current string */
-		*xp = '\0';
-		/* move the actual string forward */
-		memmove(cp + Zfc_e_dash, cp, linelen + /* NUL */ 1);
-		xp += Zfc_e_dash;
-		/* prepend it with "fc -e -" */
-		memcpy(cp, Tfc_e_dash, Zfc_e_dash);
-	}
-#endif
 	s->start = s->str = cp;
 	strip_nuls(Xstring(s->xs, xp), Xlength(s->xs, xp));
 	/* Note: if input is all nulls, this is not eof */
@@ -1514,7 +1499,7 @@ set_prompt(int to, Source *s)
 			ps1 = shf_sclose(shf);
 			saved_atemp = ATEMP;
 			newenv(E_ERRH);
-			if (sigsetjmp(e->jbuf, 0)) {
+			if (kshsetjmp(e->jbuf)) {
 				prompt = safe_prompt;
 				/*
 				 * Don't print an error - assume it has already
@@ -1633,7 +1618,7 @@ get_brace_var(XString *wsp, char *wp)
 
 					c2 = getsc();
 					ungetsc(c2);
-					if (c2 != '}') {
+					if (c2 != /*{*/ '}') {
 						ungetsc(c);
 						goto out;
 					}
