@@ -157,9 +157,9 @@
 #endif
 
 #ifdef EXTERN
-__RCSID("$MirOS: src/bin/mksh/sh.h,v 1.560 2012/05/09 23:20:58 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/sh.h,v 1.564 2012/06/26 19:33:33 tg Exp $");
 #endif
-#define MKSH_VERSION "R40 2012/05/09"
+#define MKSH_VERSION "R40 2012/06/26"
 
 /* arithmetic types: C implementation */
 #if !HAVE_CAN_INTTYPES
@@ -427,7 +427,14 @@ extern int wcwidth(__WCHAR_TYPE__);
 #define LINE		4096	/* input line size */
 
 EXTERN const char *safe_prompt; /* safe prompt if PS1 substitution fails */
-EXTERN const char initvsn[] E_INIT("KSH_VERSION=@(#)MIRBSD KSH " MKSH_VERSION);
+
+#ifdef MKSH_LEGACY_MODE
+#define KSH_VERSIONNAME	"LEGACY"
+#else
+#define KSH_VERSIONNAME	"MIRBSD"
+#endif
+EXTERN const char initvsn[] E_INIT("KSH_VERSION=@(#)" KSH_VERSIONNAME \
+    " KSH " MKSH_VERSION);
 #define KSH_VERSION	(initvsn + /* "KSH_VERSION=@(#)" */ 16)
 
 EXTERN const char digits_uc[] E_INIT("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ");
@@ -529,6 +536,14 @@ im_sorry_dave(void)
 } while (/* CONSTCOND */ 0)
 #endif
 
+#ifdef MKSH_LEGACY_MODE
+#ifndef MKSH_NO_CMDLINE_EDITING
+#define MKSH_NO_CMDLINE_EDITING	/* defined */
+#endif
+#undef MKSH_S_NOVI
+#define MKSH_S_NOVI		1
+#endif
+
 #ifdef MKSH_SMALL
 #ifndef MKSH_CONSERVATIVE_FDS
 #define MKSH_CONSERVATIVE_FDS	/* defined */
